const std = @import("std");

const Uxn = @import("uxn.zig");
const Varvara = @This();

uxn: Uxn,

allocator: std.mem.Allocator,
metadata: ?[]const u8 = null,

pub fn init(allocator: std.mem.Allocator, rom: []const u8) !Varvara {
    var uxn = try Uxn.init(allocator);
    uxn.deo8 = deo8;
    uxn.deo16 = deo16;
    uxn.dei8 = dei8;
    uxn.dei16 = dei16;
    @memcpy(uxn.mem.m[0x100 .. rom.len + 0x100], rom);
    return .{ .uxn = uxn, .allocator = allocator };
}

pub fn deinit(self: *Varvara) void {
    self.uxn.deinit(self.allocator);
}

pub fn deo8(uxn: *Uxn, stack: *Uxn.Stack, device: u8, value: u8) void {
    const self: *Varvara = @fieldParentPtr("uxn", uxn);
    _ = .{ self, stack, device, value };
    switch (device) { // Console/write
        0x18 => {
            std.io.getStdOut().writer().writeByte(value) catch |e| @panic(@errorName(e));
        },
        else => {},
    }
}

pub fn deo16(uxn: *Uxn, stack: *Uxn.Stack, device: u8, value: u16) void {
    const self: *Varvara = @fieldParentPtr("uxn", uxn);
    _ = .{ self, stack, device, value };
    switch (device) {
        0x06, 0x07 => { // System/metadata
            self.metadata = std.mem.span(@as([*:0]const u8, @ptrCast(&uxn.mem.m[value +% 1])));
        },
        else => {},
    }
}

pub fn dei8(uxn: *Uxn, stack: *Uxn.Stack, device: u8) void {
    const self: *Varvara = @fieldParentPtr("uxn", uxn);
    _ = .{ self, stack, device };
    switch (device) {
        else => {},
    }
}

pub fn dei16(uxn: *Uxn, stack: *Uxn.Stack, device: u8) void {
    const self: *Varvara = @fieldParentPtr("uxn", uxn);
    _ = .{ self, stack, device };
    switch (device) {
        else => {},
    }
}
